@@ -10,10 +10,11 @@ extern crate mime_guess;
 extern crate itertools;
 extern crate tabwriter;
 extern crate lazysort;
-extern crate brotli2;
 extern crate unicase;
 extern crate walkdir;
 extern crate base64;
+extern crate blake3;
+extern crate brotli;
 extern crate flate2;
 extern crate rfsapi;
 #[cfg(target_os = "windows")]
@@ -30,7 +31,6 @@ extern crate iron;
 extern crate libc;
 extern crate rand;
 extern crate time;
-extern crate md6;
 extern crate xml;
 
 mod error;
@@ -76,10 +76,14 @@ fn result_main() -> Result<(), Error> {
         opts.path_auth_data.insert(path, Some(ops::generate_auth_data()));
     }
 
+    let handler = ops::SimpleChain {
+        handler: ops::HttpHandler::new(&opts),
+        after: opts.request_bandwidth.map(ops::LimitBandwidthMiddleware::new),
+    };
     let mut responder = if let Some(p) = opts.port {
-        if let Some(&((ref id, _), ref pw)) = opts.tls_data.as_ref() {
-                Iron::new(ops::HttpHandler::new(&opts)).https((opts.bind_address, p),
-                                                              NativeTlsServer::new(id, pw).map_err(|err| {
+        if let Some(&((_, ref id), ref pw)) = opts.tls_data.as_ref() {
+                Iron::new(handler).https((opts.bind_address, p),
+                                         NativeTlsServer::new(id, pw).map_err(|err| {
                         Error {
                             desc: "TLS certificate",
                             op: "open",
@@ -87,7 +91,7 @@ fn result_main() -> Result<(), Error> {
                         }
                     })?)
             } else {
-                Iron::new(ops::HttpHandler::new(&opts)).http((opts.bind_address, p))
+                Iron::new(handler).http((opts.bind_address, p))
             }
             .map_err(|_| {
                 Error {
@@ -97,16 +101,14 @@ fn result_main() -> Result<(), Error> {
                 }
             })
     } else {
-        ops::try_ports(ops::HttpHandler::new(&opts),
-                       opts.bind_address,
-                       util::PORT_SCAN_LOWEST,
-                       util::PORT_SCAN_HIGHEST,
-                       &opts.tls_data)
+        ops::try_ports(handler, opts.bind_address, util::PORT_SCAN_LOWEST, util::PORT_SCAN_HIGHEST, &opts.tls_data)
     }?;
 
     if opts.loglevel < options::LogLevel::NoStartup {
-        print!("{}Hosting \"{}\" on port {}",
-               trivial_colours::Reset,
+        if opts.log_colour {
+            print!("{}", trivial_colours::Reset);
+        }
+        print!("Hosting \"{}\" on port {}",
                opts.hosted_directory.0,
                responder.socket.port());
         if responder.socket.ip() != IpAddr::from([0, 0, 0, 0]) {
@@ -125,7 +127,18 @@ fn result_main() -> Result<(), Error> {
         }
         println!("...");
 
-        if !opts.proxies.is_empty()  {
+        if let Some(band) = opts.request_bandwidth {
+            println!("Requests limited to {}B/s.", band);
+        }
+
+        for (ext, mime_type) in opts.mime_type_overrides {
+            match &ext[..] {
+                "" => println!("Serving files with no extension as {}.", mime_type),
+                _ => println!("Serving files with .{} extension as {}.", ext, mime_type),
+            }
+        }
+
+        if !opts.proxies.is_empty() {
             println!("Trusted proxies:");
 
             let mut out = TabWriter::new(stdout());
@@ -133,6 +146,10 @@ fn result_main() -> Result<(), Error> {
             for (network, header) in &opts.proxies {
                 writeln!(out, "{}\t{}", header, network).unwrap();
             }
+            writeln!(out, "URL Header\tNetwork").unwrap();
+            for (network, header) in &opts.proxy_redirs {
+                writeln!(out, "{}\t{}", header, network).unwrap();
+            }
             out.flush().unwrap();
         }
     }
@@ -168,12 +185,11 @@ fn result_main() -> Result<(), Error> {
             move || r.notify_one()
         })
         .unwrap();
-    let mx = Mutex::new(());
-    let _ = end_handler.wait(mx.lock().unwrap()).unwrap();
+    drop(end_handler.wait(Mutex::new(()).lock().unwrap()).unwrap());
     responder.close().unwrap();
 
     // This is necessary because the server isn't Drop::drop()ped when the responder is
-    ops::HttpHandler::clean_temp_dirs(&opts.temp_directory, opts.loglevel);
+    ops::HttpHandler::clean_temp_dirs(&opts.temp_directory, opts.loglevel, opts.log_colour);
 
     Ok(())
 }
